@@ -57,12 +57,14 @@ void printArray(double *a, int count) {
  */
  
 void CCM(int *nobserved, int *abridged, int *npred, double *MIGm, double *MIGf, int *migr, int *migc,
-                     int *MIGtype, double *srm, double *srf, double *asfr, double *srb, 
+                     int *MIGtype, double *MIGratem, double *MIGratef, int *MIGratecode, 
+                     double *srm, double *srf, double *asfr, double *srb, 
                      double *Lm, double *Lf, double *lxm, double *lxf,
                      int *nages, int *nfages, int *fstart, 
                      double *popm, double *popf, double *totp, 
-                     double *btagem, double *btagef, double *deathsm, double *deathsf
-) {
+                     double *btagem, double *btagef, double *deathsm, double *deathsf,
+                     double *finmigm, double *finmigf
+    ) {
      
     double b, bm, bf, srb_ratio;
     int i, j, jve, adim, adim1, nrow, ncol, n, t, t1, t_offset;
@@ -76,13 +78,17 @@ void CCM(int *nobserved, int *abridged, int *npred, double *MIGm, double *MIGf,
     int adimfert_start = *fstart - 1; /* Start index of reproductive age (from R to C) */
     
     double cdeathsm[adim], cdeathsf[adim], bt[adimfert];
-    double migm[adim][*migc], migf[adim][*migc];
+    double migm[adim][*migc], migf[adim][*migc]; /*migration counts */
     double popadjm[adim][*npred+1], popadjf[adim][*npred+1], current_popf;
     double migendm[adim][*migc], migendf[adim][*migc], migmidm[adim][*migc], migmidf[adim][*migc];
     double migstartm[adim][*migc], migstartf[adim][*migc];
+    double totmigcount, tpopm, tpopf, trmig, trmigpos, trmigneg, tmp;
     
     double csfm[adim],csff[adim]; /* cohort separation factor males, females*/
     
+    const double minpop = 0.0005; /* minimum accepted population */
+    const double max_out_rate = -0.8; /* the maximum portion of population that can leave when using migration rates */ 
+    
     nrow = *migr;
     ncol = *migc;
     n = *npred;
@@ -91,6 +97,18 @@ void CCM(int *nobserved, int *abridged, int *npred, double *MIGm, double *MIGf,
         Rprintf("\nadim = %i adimfert = %i adimfert_start = %i migc = %i npred = %i adimdif = %i", 
                 adim, adimfert, adimfert_start, *migc, n, adimdif);
     
+    /* If migration is given as rates in MIGratem and MIGratef, then migm and migf
+     * are assumed to contain the age-schedules.
+     * 
+     * MIGratecode:
+     *      0: migration given as counts in MIGm and MIGf
+     *      > 0: migration given as total rates in MIGratem and MIGratef,
+     *              while MIGm and MIGf contains the age schedules.
+     *      1: the age schedules are proportions
+     *      2: the age schedules are totals (used for schedules that have positive as well as negative parts)
+     *      3: the age schedules are the actual final age-specific rates (i.e. rate * schedule)
+     */
+    
     for(j=0; j<ncol; ++j) {
         /* If migration has less age groups then population, fill-in the remaining age groups with 0 */
         for(i=0; i<nrow; ++i) {		
@@ -103,36 +121,50 @@ void CCM(int *nobserved, int *abridged, int *npred, double *MIGm, double *MIGf,
         }
         /* Migration is going to be added to population at the start (for computing deaths), 
          * middle (for computing births) and at the end of the interval.
-         * Migration type determines what is added when.
+         * Migration type (MIGtype) determines what is added when.
+         * If migration is given as rates, counts are computed and added at the end of the interval.
          */
-        switch (*MIGtype) { 
-        case 0: /* migration evenly distributed over each interval (MigCode=0) */
-            for(i=0; i<nrow+adimdif; ++i) { /* half at the start, half in the middle, nothing at the end*/
-                migstartm[i][j] = 0.5 * migm[i][j];
-                migstartf[i][j] = 0.5 * migf[i][j];
-                migmidm[i][j] = migstartm[i][j];
-                migmidf[i][j] = migstartf[i][j];
-                migendm[i][j] = 0;
-                migendf[i][j] = 0;
+        if(MIGratecode[j] == 0){ /* migration is given as counts */
+            switch (*MIGtype) { 
+            case 0: /* migration evenly distributed over each interval (MigCode=0) */
+                for(i=0; i<nrow+adimdif; ++i) { /* half at the start, half in the middle, nothing at the end*/
+                    migstartm[i][j] = 0.5 * migm[i][j];
+                    migstartf[i][j] = 0.5 * migf[i][j];
+                    migmidm[i][j] = migstartm[i][j];
+                    migmidf[i][j] = migstartf[i][j];
+                    migendm[i][j] = 0;
+                    migendf[i][j] = 0;
+                }
+                break;
+            default: /* migration at the end of each interval (MigCode=9)*/
+                for(i=0; i<nrow+adimdif; ++i) { /* everything is added at the end */
+                    migstartm[i][j] = 0;
+                    migstartf[i][j] = 0;
+                    migmidm[i][j] = 0;
+                    migmidf[i][j] = 0;
+                    migendm[i][j] = migm[i][j];
+                    migendf[i][j] = migf[i][j];
+                }
+                break;
             }
-            break;
-        default: /* migration at the end of each interval (MigCode=9)*/
-            for(i=0; i<nrow+adimdif; ++i) { /* everything is added at the end */
+        } else { /* migration is given as rates */
+            for(i=0; i<nrow+adimdif; ++i) { 
                 migstartm[i][j] = 0;
                 migstartf[i][j] = 0;
                 migmidm[i][j] = 0;
                 migmidf[i][j] = 0;
-                migendm[i][j] = migm[i][j];
+                migendm[i][j] = migm[i][j]; /* these are now age schedules, depending on MIGratecode */
                 migendf[i][j] = migf[i][j];
             }
-            break;
         }
     }
-    
+
+ 
+
     /* Population projection for one trajectory */
     for(j=1; j<(n+1); ++j) {
         jve = j-1;
-        if(debug==3) Rprintf("\nj = %i, jve = %i", j, jve);
+        if(debug==1) Rprintf("\nj = %i, jve = %i", j, jve);
         t = j*adim;
         t1 = (j-1)*adim; /* used to access the previous time period */
         t_offset = jve*adim; /* although the same as t1, using a different name for clarity, see comment below */
@@ -198,16 +230,78 @@ void CCM(int *nobserved, int *abridged, int *npred, double *MIGm, double *MIGf,
             popm[t] = bm - cdeathsm[0] + migmidm[0][jve];
             popf[t] = bf - cdeathsf[0] + migmidf[0][jve];
 
+            /* If migration is a rate, compute the migration counts */
+            if(MIGratecode[jve] > 0){
+                trmig = 0;
+                trmigpos = 0;
+                trmigneg = 0;
+                if(MIGratecode[jve] == 2){ /* for schedules that have positive and negative parts get the sum of these */
+                    for(i=0; i < adim; ++i) {  
+                        trmig = trmig + migendm[i][jve] + migendf[i][jve];
+                        trmigpos = trmigpos + fmax(0, migendm[i][jve]) + fmax(0, migendf[i][jve]);
+                        trmigneg = trmigneg + fmin(0, migendm[i][jve]) + fmin(0, migendf[i][jve]);
+                    }
+                }
+                
+                if(MIGratecode[jve] < 3){ /* need to disaggregate into ages */
+                    /* sum population together */
+                    tpopm = 0; tpopf = 0;
+                    for(i=0; i < adim; ++i) {
+                        tpopm = tpopm + popm[i + t];
+                        tpopf = tpopf + popf[i + t];
+                    }
+                    /* total migration count; not more than 80% of total population can leave */ 
+                    totmigcount = fmax(MIGratem[jve], max_out_rate) * tpopm + fmax(MIGratef[jve], max_out_rate) * tpopf; 
+                    tmp = 0;
+                    /* distribute into ages */
+                    for(i=0; i < adim; ++i) { 
+                        if(MIGratecode[jve] == 1){ /* multiply the total rate with age schedule */
+                            migendm[i][jve] = fmax(fmax(0, popm[i+t])*max_out_rate, totmigcount*migendm[i][jve]); /* assures there is no depopulation */
+                            migendf[i][jve] = fmax(fmax(0, popf[i+t])*max_out_rate, totmigcount*migendf[i][jve]);
+                            tmp = tmp + migendm[i][jve];
+                        }
+                        if(MIGratecode[jve] == 2){ /* these schedules are actual counts, so we shift them up and down, relative to the sum */
+                            if(totmigcount > 0){
+                                if(migendm[i][jve] > 0) /* distribute the difference across the positive part of the schedule */
+                                    migendm[i][jve] = migendm[i][jve] + (totmigcount - trmig)*migendm[i][jve]/trmigpos;
+                                if(migendf[i][jve] > 0) 
+                                    migendf[i][jve] = migendf[i][jve] + (totmigcount - trmig)*migendf[i][jve]/trmigpos;
+                            } else { /* rate is negative */
+                                if(migendm[i][jve] < 0) /* distribute the difference across the negative part of the schedule */
+                                    migendm[i][jve] = migendm[i][jve] + (totmigcount - trmig)*migendm[i][jve]/trmigneg;
+                                if(migendf[i][jve] < 0) 
+                                    migendf[i][jve] = migendf[i][jve] + (totmigcount - trmig)*migendf[i][jve]/trmigneg;   
+                            }
+                        }
+                    }
+                    if(debug>=1 && jve > 9 && totmigcount > 0){
+                        Rprintf("\ntotmigcount = %f, tpopm = %f, tpopf = %f, sum(migendm) = %f, migendm[5] = %f, migendm[10] = %f, migendf[10] = %f",  
+                                totmigcount, tpopm, tpopf, tmp, migendm[5][jve], migendm[10][jve], migendf[10][jve]);
+                    }
+                } else { /* MIGratecode[jve] == 3; the schedules are the final age-specific rates */
+                    for(i=0; i < adim; ++i) { 
+                        migendm[i][jve] = fmax(fmax(0,popm[i+t])*max_out_rate, migendm[i][jve]);
+                        migendf[i][jve] = fmax(fmax(0,popf[i+t])*max_out_rate, migendf[i][jve]);
+                    }
+                }
+            }
+            
             /* add migration at the end of the interval, adjust negative population and compute total pop */
             totp[j] = 0;
             for(i=0; i < adim; ++i) {
+                migendm[i][jve] = fmax(migendm[i][jve], -popm[i+t] + minpop); /* adjust migration if it would yield negative population */
                 popm[i+t] = popm[i + t] + migendm[i][jve];
+                migendf[i][jve] = fmax(migendf[i][jve], -popf[i+t] + minpop);
                 popf[i+t] = popf[i + t] + migendf[i][jve];
-                popm[i+t] = fmax(popm[i + t], 0.0005);
-                popf[i+t] = fmax(popf[i + t], 0.0005);
-                totp[j] += popm[i + t]+popf[i + t]; 
+                totp[j] += popm[i + t] + popf[i + t]; 
             }
         }
+        
+        for(i=0; i < nrow; ++i) {
+            finmigm[i + jve*nrow] = migstartm[i][jve] + migmidm[i][jve] + migendm[i][jve];
+            finmigf[i + jve*nrow] = migstartf[i][jve] + migmidf[i][jve] + migendf[i][jve];
+        }
+        
         /**************************************************************************/
         /* period deaths                                                          */
         /* 1. calculated cohort separation factors                                */
